package de.akquinet.jbosscc.needle.seam;

import de.akquinet.jbosscc.needle.inject.Injector;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.log.Log;

import java.lang.reflect.Field;
import java.util.List;

/**
 * @author: bendt
 */
public class SeamInjector extends Injector {
    public List<Field> getAllSeamInjectedFields(final Object instance) {
        return getAllFieldsWithAnnotation(instance, In.class);
    }

    /**
     * Injects a given mock object to all fields with annotation \@Logger .
     *
     * @param target -- target object of injection
     * @param log    -- object that is injected
     */
    public final void injectLogger(final Object target, final Log log) {
        injectAllDeclaredFields(target, Logger.class, log);
    }

    /**
     * Injects a predefined object to all fields with annotation \@Logger .
     *
     * @param target -- target object of injection
     */
    public final MockLog injectSeamLogger(final Object target) {
        MockLog logMock = new MockLog();
        injectLogger(target, logMock);
        return logMock;
    }

    /**
     * Injects on the field with a given name and which is annotated with \@In .
     * annotation of Seam
     *
     * @param injectionTarget -- target object of injection
     * @param name            -- name of the target field
     * @param injectedObject  -- object that is injected
     */
    public final void injectSeamIn(final Object injectionTarget, final String name, final Object injectedObject) {
        assert injectedObject != null;
        assert name != null;

        final AnnotationChecker<In> seamInChecker = new AnnotationChecker<In>() {

            public boolean checkAnnotation(final Field field, final In annotation) {
                boolean isOK = false;
                if (In.class.isInstance(annotation)) {
                    final In inAnnotation = In.class.cast(annotation);
                    isOK = (name.equals(inAnnotation.value()) || name.equals(field.getName()));
                }
                return isOK;
            }

        };
        injectAllDeclaredFields(injectionTarget, In.class, injectedObject, seamInChecker);
    }

    /**
     * Injects an object to the field with a given name and with annotation
     * \@RequestParameter .
     *
     * @param injectionTarget      -- target object of injection
     * @param requestParameterName -- name of the target field
     * @param object               -- object that is injected
     */
    public final void injectRequestParameter(final Object injectionTarget, final String requestParameterName,
                                             final Object object) {
        injectAllDeclaredFields(injectionTarget, RequestParameter.class, object,
                new AnnotationChecker<RequestParameter>() {

                    public boolean checkAnnotation(final Field field, final RequestParameter annotation) {
                        return annotation.value().equals(requestParameterName);
                    }
                });
    }

}
